from __future__ import print_function, absolute_import, unicode_literals

import logging
from logging.handlers import RotatingFileHandler
from os import makedirs
from os.path import join, exists, dirname
import sys

from clyent import errors
from clyent.colors import initialize_colors

from .handlers import ColorStreamHandler


def log_unhandled_exception(logger):
    def excepthook(*exc_info):
        logger.error('', exc_info=exc_info)
        sys.exit(1)

    return excepthook

def setup_logging(logger, level, use_color=None, logfile=None, show_tb=False):
    initialize_colors(use_color)

    logger.setLevel(logging.DEBUG)

    cli_logger = logging.getLogger('cli-logger')
    cli_logger.setLevel(logging.ERROR)
    if logfile:
        if not exists(dirname(logfile)): makedirs(dirname(logfile))
        hndlr = RotatingFileHandler(logfile, maxBytes=10 * (1024 ** 2), backupCount=5,)
        hndlr.setLevel(logging.ERROR)
        fmt = logging.Formatter("[%(asctime)s] %(message)s")
        hndlr.setFormatter(fmt)

        logger.addHandler(hndlr)

        cli_logger.addHandler(hndlr)

    exceptions = (errors.ClyentError, KeyboardInterrupt)
    shndlr = ColorStreamHandler(show_tb=show_tb, exceptions=exceptions)
    shndlr.setLevel(level)
    logger.addHandler(shndlr)

    sys.excepthook = log_unhandled_exception(logger)




from __future__ import unicode_literals
from argparse import ArgumentParser
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import os
import sys
import unittest

import mock

from clyent import add_subparser_modules
from clyent.colors.color_formatter import print_colors
from clyent.colors import Color, ColorStream

def add_hello_parser(subparsers):
    subparser = subparsers.add_parser('hello')
    subparser.add_argument('world')
    subparser.set_defaults(main=mock.Mock())

class Test(unittest.TestCase):

    def test_add_subparser_modules(self):
        parser = ArgumentParser()

        with mock.patch('clyent.iter_entry_points') as iter_entry_points:

            ep = mock.Mock()
            ep.load.return_value = add_hello_parser
            iter_entry_points.return_value = [ep]
            add_subparser_modules(parser, None, 'entry_point_name')

        args = parser.parse_args(['hello', 'world'])
        self.assertEqual(args.world, 'world')

    @unittest.skipIf(os.name == 'nt', 'Cannot colorize StringIO on Windows')
    def test_color_format(self):
        output = StringIO()
        output.fileno = lambda: -1
        stream = ColorStream(output)

        print_colors('Are you', '{=okay!c:green}', 'Annie?', file=stream)

        value = output.getvalue()
        output.close()

        self.assertEqual('Are you \033[92mokay\033[0m Annie?\n', value)

    @unittest.skipIf(os.name == 'nt', 'Cannot colorize StringIO on Windows')
    def test_color_context(self):
        output = StringIO()
        output.fileno = lambda: -1
        stream = ColorStream(output)

        with Color('red', stream):
            print_colors('ERROR!', file=stream)

        value = output.getvalue()
        output.close()

        self.assertEqual('\033[91mERROR!\n\033[0m', value)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, print_function, unicode_literals

import logging
import sys

from clyent import colors, errors


COLOR_MAP = {logging.ERROR: 'red',
             logging.WARN: 'yello',
             logging.DEBUG: 'blue'}

class ColorStreamHandler(logging.Handler):

    def __init__(self, level=logging.INFO, show_tb='tty', exceptions=(errors.ClyentError,)):
        self.show_tb = show_tb
        self.exceptions = exceptions

        logging.Handler.__init__(self, level=level)


    def emit(self, record):

        if not self.filter(record):
            return

        msg = self.format(record)

        if record.levelno <= logging.INFO:
            stream = sys.stdout
        else:
            stream = sys.stderr

        color = colors.Color(COLOR_MAP.get(record.levelno), file=stream)

        is_hidable = record.exc_info and isinstance(record.exc_info[1], self.exceptions)
        should_hide = True if self.show_tb == 'never' else False if self.show_tb == 'always' else stream.isatty()

        if is_hidable and should_hide:
            err = record.exc_info[1]
            msg = str(err)
            with color:
                print('[%s] ' % type(err).__name__, file=stream, end='')

            print(msg, file=stream)


        else:
            header = record.levelname if record.levelname != 'INFO' else ''
            if header.strip():
                with color:
                    print('[%s] ' % header, file=stream, end='')

            print(msg, file=stream)
            
        stream.flush()


def main():

    colors.initialize_colors()
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    h = ColorStreamHandler(logging.DEBUG, show_tb=False)
    logger.addHandler(h)

    logger.debug("DEBUG")
    logger.info("INFO")
    logger.warn("WARN")
    logger.error("ERROR")

#     FormatterWrapper.wrap(h, prefix=color('prefix |', [color.WHITE, color.BACKGROUND_COLORS[0]]))
    try:
        asdf
    except:
        logger.exception("Show this tb")

    try:
        raise errors.ClyentError("This will be a short message")
    except:
        logger.exception("This wil not be displayed")



if __name__ == '__main__':
    main()
